_G.NSSU = NSSU or {}
NSSU._mod_path = ModPath
NSSU._default_localization_path = NSSU._mod_path .. "l10n/en.json"
NSSU.main_menu_id = "menu_nssu"
NSSU._settings_path = SavePath .. "noscreenshakeupdated_settings.json"
NSSU.default_settings = {
	shaker_disable_all = false -- disables ALL shakers
}
NSSU._shaker_setting_map = {} --lookup table for setting_name:shaker_name
NSSU.shaker_blacklist = { -- determines whether a given shader is blocked
	fire_weapon_rot = true,
	fire_weapon_kick = true,
	breathing = true,
	headbob = true,
	mission_triggered = true,-- includes explosions such as player throwables
	player_start_running = true,
	player_taser_shock = true,
	player_bullet_damage = true,
	whizby = true,
	player_bleedout_land = true,
	player_land = true,
	player_fall_damage = true,
	player_melee_charge_fist = true,
	player_melee_charge_wing = true,
	player_melee_charge = true,
	player_melee = true,
	player_melee_var2 = true,
	player_enter_zipline = true,
	player_on_zipline = true,
	player_exit_zipline = true,
	cash_opening = false
}
NSSU._shaker_blacklist_order = { -- order for menu object creation
	"fire_weapon_rot",
	"fire_weapon_kick",
	"breathing",
	"headbob",
	"mission_triggered",
	"player_start_running",
	"player_taser_shock",
	"player_bullet_damage",
	"whizby",
	"player_bleedout_land",
	"player_land",
	"player_fall_damage",
	"player_melee_charge_fist",
	"player_melee_charge_wing",
	"player_melee_charge",
	"player_melee",
	"player_melee_var2",
	"player_enter_zipline",
	"player_on_zipline",
	"player_exit_zipline",
	"cash_opening"
}
NSSU.shaker_parameter_overrides = {
	headbob = {
		amplitude = 0
	},
	breathing = {
		amplitude = 0
	}
}

-- generate settings from blacklist
for shaker_name,state in pairs(NSSU.shaker_blacklist) do 
	local setting_name = "shaker_disable_" .. shaker_name
	NSSU.default_settings[setting_name] = state
	NSSU._shaker_setting_map[setting_name] = shaker_name
end
NSSU.settings = table.deep_map_copy(NSSU.default_settings)






-- ======== Blacklist table management

function NSSU:upd_shaker_blacklist()
	for setting_name,state in pairs(self.settings) do 
		local shaker_name = self._shaker_setting_map[setting_name]
		if shaker_name then 
			self.shaker_blacklist[shaker_name] = state
		end
	end
end

function NSSU:on_shaker_setting_changed(setting_name,do_shaker)
	local shaker_name = self._shaker_setting_map[setting_name]
	if shaker_name then 
		local state = self.settings[setting_name]
		self:_on_shaker_setting_changed(shaker_name,state,do_shaker)
	end
end

function NSSU:_on_shaker_setting_changed(shaker_name,state,do_shaker)
	self.shaker_blacklist[shaker_name] = state
	if do_shaker then 
		-- stop relevant shaker
		local player = managers.player:local_player()
		if state and alive(player) then 
			local camera = player:camera()
			if camera then 
				camera:stop_shaker(shaker_name)
			end
		end
	end
end


-- ======== I/O

function NSSU:Load()
	local file = io.open(self._settings_path, "r")
	if file then
		for k, v in pairs(json.decode(file:read("*all"))) do
			self.settings[k] = v
		end
	end
end

function NSSU:Save()
	local file = io.open(self._settings_path,"w+")
	if file then
		file:write(json.encode(self.settings))
		file:close()
	end
end

Hooks:Add("LocalizationManagerPostInit","LocalizationManagerPostInit_nssu",function(loc_manager)
	if not BeardLib then
		loc_manager:load_localization_file(NSSU._default_localization_path)
	end
end)

-- ======== Menu Generation

-- Hooks:Add("MenuManagerInitialize","MenuManagerInitialize_nssu",function(menu_manager) end)

Hooks:Add("MenuManagerSetupCustomMenus","MenuManagerSetupCustomMenus_nssu",function(menu_manager,nodes)
	MenuHelper:NewMenu(NSSU.main_menu_id)
end)

Hooks:Add("MenuManagerPopulateCustomMenus","MenuManagerPopulateCustomMenus_nssu",function(menu_manager,nodes)
	NSSU:Load()
	NSSU:upd_shaker_blacklist()
	
	local menu_id = NSSU.main_menu_id
	
	local i = 1
	for j=#NSSU._shaker_blacklist_order,1,-1 do
		local shaker_name = NSSU._shaker_blacklist_order[j]
		local setting_name = "shaker_disable_" .. shaker_name
		local callback_name = "callback_nssu_" .. setting_name
		MenuCallbackHandler[callback_name] = function(self,item)
			local state = item:value() == "on"
			NSSU.settings[setting_name] = state
			NSSU:_on_shaker_setting_changed(shaker_name,state,true)
			NSSU:Save()
		end
		
		MenuHelper:AddToggle({
			id = "nssu_" .. setting_name,
			title = "menu_nssu_" .. setting_name .. "_title",
			desc = "menu_nssu_" .. setting_name .. "_desc",
			callback = callback_name,
			value = NSSU.settings[setting_name],
			menu_id = menu_id,
			priority = i
		})
		i = i + 1
	end
	
	MenuHelper:AddButton({
		id = "nssu_header",
		title = "menu_nssu_header_title",
		desc = "menu_nssu_header_desc",
		--callback = "callback_nssu_toggle_disable_all",
		disabled = true,
		menu_id = menu_id,
		priority = i
	})
	i = i + 1
	
	MenuHelper:AddDivider({
		id = "nssu_div_1",
		size = 12,
		menu_id = menu_id,
		priority = i
	})
	i = i + 1
	
	MenuCallbackHandler.callback_nssu_toggle_disable_all = function(self,item)
		local state = item:value() == "on"
		NSSU.settings.shaker_disable_all = state
		NSSU:_on_shaker_setting_changed("shaker_disable_all",state)
		NSSU:Save()
	end
	
	MenuHelper:AddToggle({
		id = "nssu_toggle_shaker_disable_all",
		title = "menu_nssu_toggle_shaker_disable_all_title",
		desc = "menu_nssu_toggle_shaker_disable_all_desc",
		callback = "callback_nssu_toggle_disable_all", --intentionally break from the naming convention of the other shaker settings... just in case there's a shaker called "all" or something
		value = NSSU.settings.shaker_disable_all,
		menu_id = menu_id,
		priority = i
	})
	
end)

Hooks:Add("MenuManagerBuildCustomMenus","MenuManagerBuildCustomMenus_nssu",function(menu_manager,nodes)
	-- create nssu menu
	local menu_id = NSSU.main_menu_id
	local nssu_menu = MenuHelper:BuildMenu(
		menu_id,{
			area_bg = "none",
			back_callback = nil,
			focus_changed_callback = nil
		}
	)
	nodes[menu_id] = nssu_menu
	MenuHelper:AddMenuItem(nodes.blt_options,menu_id,"menu_nssu_title","menu_nssu_desc")
end)